// Copyright 1997 The Open Group Research Institute.  All rights reserved.

package krb5.lib.crypto;

import krb5.lib.*;
//XXX only needed if dataSize() implementation changes back to spec;
//see dataSize() below
//import krb5.lib.asn1.*;

public abstract class EType {

	public static EType getInstance(int eTypeConst)
		throws KdcErrException {
		EType eType = null;
		switch (eTypeConst) {
			case Krb5.ETYPE_NULL:
				eType = new NullEType();
				break;
			case Krb5.ETYPE_DES_CBC_CRC:
				eType = new DesCbcCrcEType();
				break;
			case Krb5.ETYPE_DES_CBC_MD4:
				eType = new DesCbcMd4EType();
				break;
			case Krb5.ETYPE_DES_CBC_MD5:
				eType = new DesCbcMd5EType();
				break;
			default:
				throw new KdcErrException(Krb5.KDC_ERR_ETYPE_NOSUPP);
		}
		return eType;
	}

	public static EType getInstance() throws KdcErrException {
		return getInstance(Config.ETYPE_DEFAULT);
	}

	public abstract int eType();

	public abstract int minimumPadSize();

	public abstract int confounderSize();

	public abstract int checksumType();

	public abstract int checksumSize();

	public abstract int blockSize();

	public abstract int keyType();

	public abstract int keySize();

	public abstract byte[] encrypt(byte[] data, byte[] key);

	public abstract byte[] encrypt(byte[] data, byte[] key, byte[] ivec);

	public abstract void decrypt(byte[] cipher, byte[] key)
		throws KrbApErrException;

	public abstract void decrypt(byte[] cipher, byte[] key, byte[] ivec)
		throws KrbApErrException;

	public abstract byte[] calculateChecksum(byte[] data, int size);

	public int dataSize(byte[] data)
		//throws Asn1Exception
	{
		//EncodeRef ref = new EncodeRef(data, startOfData());
		//return ref.end - startOfData();
		//XXX should be the above according to spec, but in fact
		//implementations include the pad bytes in the data size
		return data.length - startOfData();
	}

	public int padSize(byte[] data) {
		return data.length - confounderSize() - checksumSize() -
			dataSize(data);
	}

	public int startOfChecksum() {
		return confounderSize();
	}

	public int startOfData() {
		return confounderSize() + checksumSize();
	}

	public int startOfPad(byte[] data) {
		return confounderSize() + checksumSize() + dataSize(data);
	}

	public byte[] decryptedData(byte[] data) {
		int tempSize = dataSize(data);
		byte[] result = new byte[tempSize];
		System.arraycopy(data, startOfData(), result, 0, tempSize);
		return result;
	}

	public void copyChecksumField(byte[] data, byte[] cksum) {
		for (int i = 0; i < checksumSize();  i++)
			data[startOfChecksum() + i] = cksum[i];
	}

	public byte[] checksumField(byte[] data) {
		byte[] result = new byte[checksumSize()];
		for (int i = 0; i < checksumSize(); i++)
			result[i] = data[startOfChecksum() + i];
		return result;
	}

	public void resetChecksumField(byte[] data) {
		for (int i = startOfChecksum(); i < startOfChecksum() +
			checksumSize();  i++)
			data[i] = 0;
	}

	public void setChecksum(byte[] data, int size) {
		resetChecksumField(data);
		byte[] cksum = calculateChecksum(data, size);
		copyChecksumField(data, cksum);
	}

	public byte[] generateChecksum(byte[] data) {
		byte[] cksum1 = checksumField(data);
		resetChecksumField(data);
		byte[] cksum2 = calculateChecksum(data, data.length);
		copyChecksumField(data, cksum1);
		return cksum2;
	}

	public boolean isChecksumEqual(byte[] cksum1, byte[] cksum2) {
		if (cksum1 == cksum2)
			return true;
		if ((cksum1 == null && cksum2 != null) ||
			(cksum1 != null && cksum2 == null))
			return false;
		if (cksum1.length != cksum2.length)
			return false;
		for (int i = 0; i < cksum1.length; i++)
			if (cksum1[i] != cksum2[i])
				return false;
		return true;
	}

	public boolean isChecksumValid(byte[] data) {
		byte[] cksum1 = checksumField(data);
		byte[] cksum2 = generateChecksum(data);
		return isChecksumEqual(cksum1, cksum2);
	}

    public static String toString(int etype) {
        String result = null;
        switch (etype) {
	        case Krb5.ETYPE_NULL:
	            result = "NULL";
	            break;
	        case Krb5.ETYPE_DES_CBC_CRC:
	            result = "DES_CBC_CRC";
	            break;
	        case Krb5.ETYPE_DES_CBC_MD4:
	            result = "DES_CBC_MD4";
	            break;
	        case Krb5.ETYPE_DES_CBC_MD5:
	            result = "DES_CBC_MD5";
	            break;
	        case Krb5.ETYPE_DES3_CBC_MD5:
	            result = "DES3_CBC_MD5";
	            break;
	        case Krb5.ETYPE_DES3_CBC_SHA1:
	            result = "DES3_CBC_SHA1";
	            break;
	        case Krb5.ETYPE_DSA_WITH_SHA1_CMS_OID:
	            result = "DSA_WITH_SHA1_CMS_OID";
	            break;
	        case Krb5.ETYPE_MD5_WITH_RSA_ENCRYPTION_CMS_OID:
	            result = "MD5_WITH_RSA_ENCRYPTION_CMS_OID";
	            break;
	        case Krb5.ETYPE_SHA1_WITH_RSA_ENCRYPTION_CMS_OID:
	            result = "SHA1_WITH_RSA_ENCRYPTION_CMS_OID";
	            break;
	        case Krb5.ETYPE_RC2_CBC_ENV_OID:
	            result = "RC2_CBC_ENV_OID";
	            break;
	        case Krb5.ETYPE_RSA_ENCRYPTION_ENV_OID:
	            result = "RSA_ENCRYPTION_ENV_OID";
	            break;
	        case Krb5.ETYPE_RSA_ES_OAEP_ENV_OID:
	            result = "RSA_ES_OAEP_ENV_OID";
	            break;
	        case Krb5.ETYPE_DES_EDE3_CBC_ENV_OID:
	            result = "DES_EDE3_CBC_ENV_OID";
	            break;
	        case Krb5.ETYPE_PK_CROSS:
	            result = "PK_CROSS";
	            break;
	        default:
	            result = Integer.toString(etype);
	    }
	    return result;
    }
}
